/*
   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA
*/


#include <ctype.h>
#include "core/memaid.h"
#include "defs.h"

#define CGI_STATE_FILE  pathed("api_state.bin")
#define PATH_TO_DATA_DIR "data/"
#define GRADE_THRESHOLD 3

static u_short state = 0;
/* 
  0 - idle
  1 - scheduled repetitions
  2 - new elements from queue.txt
//  3 - new element (not yet implemented)
  4 - force ahead-of-time repetitions
  5 - final drill
  6 - [not used/reserved]
//  7 - edit element (not yet implemented)
//  8 - delete element
  9 - show answer
 10 - grade
*/

static u_short answer = 0; /* if 1 then answer is already showed */
static u_short grade = 0;

#define OKA_BUT 1
#define STO_BUT 2
#define GRA_BUT 3
#define REP_BUT 4
#define NEW_BUT 5
#define DRI_BUT 6

#define CHUNKSIZE 4096
#define MAX_INT_LEN 16

static struct elem *el;
static char q[MAX_Q_LEN], a[MAX_A_LEN], hint[MAX_A_LEN], scrap[MAX_A_LEN];
static char *q_rv = NULL, *a_rv = NULL; /* received q and a */

void after_grade (u_short grade);


static void print_header (void) {
	/* puts() adds new line itself so there are two '\n' */
	puts ("Content-type: text/html\n");
}	


/* replace_all() is from my Aine project 
   - David */
void replace_all (register char *pos, const char *strfrom, const char *strto)
{
	register int len, len2;

	if (!pos || !strfrom || !strto || strlen (pos) < (len = (int)strlen(strfrom)))
		return;

	len2 = (int)strlen (strto);
	
	do {
		if ((pos = strstr (pos, strfrom)) == NULL)
			return;
		if (len > len2) {
			memmove (pos + len2, pos + len, strlen (pos + len) + 1);
		}
		else if (len < len2) {
			char *tmp = pos + len;
			memmove (tmp + (len2 - len), tmp, strlen (tmp) + 1);
		}
		strncpy (pos, strto, len2);
		pos += len2; /* len2 may be equal to 0, but it's not a problem here */
		if (*pos == '\0') return;
	} while (1);
}

int interpret_incoming_data (char *input) {
	int query_len;
	char *ps;

	if (getenv ("CONTENT_LENGTH")) {
		query_len = atoi (getenv ("CONTENT_LENGTH"));
		if (query_len > MAX_A_LEN+MAX_Q_LEN+100)
			query_len = 0;
	}
	else
		query_len = 0;
       
	if (query_len) {
		if (!(input = (char*) malloc (sizeof(char) * (query_len + 1)))) {
			ma_error ("Out of memory in CGI...\n");
			return -1;
		}
		fgets (input, query_len, stdin);
	//	replace_html_codes (input);
	}
	if (query_len && strstr (input, "question=")) {
		input += strlen ("question=");
		q_rv = input;
		if ((ps=strstr (q_rv, "&answer="))) {
			*ps = '\0';
			input = ps + strlen ("&answer=");
		}
		a_rv = input;
		if ((ps=strchr (a_rv, '&'))) {
			*ps = '\0';
			input++;
		}
		if      (strncmp (input, "oky", 3) == 0)
			return OKA_BUT;
		else if (strncmp (input, "sto", 3) == 0)
			return STO_BUT;
		else if (strncmp (input, "gr", 2) == 0) {
			grade = input[2] - '0';
			assert (grade < 6);
			return GRA_BUT;
		}
		else if (strncmp (input, "rep", 3) == 0)
			return REP_BUT;
		else if (strncmp (input, "new", 3) == 0)
			return NEW_BUT;
		else if (strncmp (input, "dri", 3) == 0)
			return DRI_BUT;
//		else if (strncmp (input, "edi", 3) == 0)
//			return 7;
//		else if (strncmp (input, "del", 3) == 0)
//			return 8;
		else
			return 0;
	}
	return 0;
}

void cgi_init (void) {
	memset (q, '\0', MAX_Q_LEN);
	memset (a, '\0', MAX_A_LEN);
	state = 0;
	answer = 0;
	print_header();
}

int main (void) {
	int i;
	char *input = NULL;
	static FILE *fskel, *fstate;

	cgi_init();
	
	i = ma_init(PATH_TO_DATA_DIR);
	if (i & DIR_NOT_ACCESSIBLE)
		exit (1);
        if (i & MEMAID_ALREADY_RUNNING)
		exit (1); /* error msg already printed by ma_init() */

	fskel = fopen (pathed ("skel.txt"), "r");
	if (!fskel) {
		ma_error ("Cannot find/open %s\n", pathed("skel.txt"));
		ma_deinit();
		exit(1);
	}
	
	if ((fstate = fopen (CGI_STATE_FILE, "rb"))) {
		if (fread (&state, sizeof(u_short), 1, fstate) != 1) {
			puts ("An error occured while reading data from cgi_state.bin file");
			exit (1);
		}
		fread (&answer, sizeof(u_short), 1, fstate);
		if (fread (&i, sizeof(u_short), 1, fstate) == 1) {
			fread (q, sizeof(char), i, fstate);
			fread (&i, sizeof(u_short), 1, fstate);
			fread (a, sizeof(char), i, fstate);
		}
		else {
			puts ("An error occured while reading data from cgi_state.bin file");
			exit (1);
		}
		fclose (fstate);
		el = ma_find_el (q, a);
		assert (el);
		remove (CGI_STATE_FILE); // are u sure ?
	}

	switch (interpret_incoming_data (input)) {
		case 0:
			break;
		case OKA_BUT:
			if (state) {
				strcpy (q, el->q);
				strcpy (a, el->a);
				strcpy (hint, "Your grade?");
				answer = 1;
			}
			else
				puts ("uhm");
			break;
		case STO_BUT:
			state = 0;
			break;
		case GRA_BUT:
			if (answer == 0) {
				break;
			}
			after_grade (grade);
			answer = 0;
			break;
		case REP_BUT:
			if ((el = ma_el_to_repeat (MA_NOT_FORCED)) != NULL) {
				strcpy (q, el->q);
				strcpy (hint, "Press OK to see the answer");
				state = 1;
			}
			else {
				state = 0;
				strcpy (hint, "Nothing is scheduled to repeat for today");
			}
			break;
		case NEW_BUT:
			// first you should intercept q/a then, answer (on the second fire up
			ma_make_new_el (grade, q_rv, a_rv);
		case DRI_BUT:
			////
			break;
	}	

	while (fgets (scrap, CHUNKSIZE, fskel)) {
		replace_all (scrap, "<hint>",  hint);
		replace_all (scrap, "<question>", q);
		replace_all (scrap, "<answer>",   a);
		fputs (scrap, stdout);
	}
	
	if (state) {
		fstate = fopen (CGI_STATE_FILE, "wb");
		fwrite (&state, sizeof(u_short), 1, fstate);
		i = strlen (el->q) + 1;
		fwrite (&i, sizeof(u_short), 1, fstate);
		fwrite (el->q, sizeof(char), i, fstate);
		i = strlen (el->a) + 1;
		fwrite (&i, sizeof(u_short), 1, fstate);
		fwrite (el->a, sizeof(char), i, fstate);
		fclose (fstate);
	}

	ma_deinit();
	if (input)
		free (input);
	fclose (fskel);
	return 0;
}

void after_grade (u_short grade) {

	if (state == 0) {
		puts ("Grade to what?");
		return;
	}
		
	switch (state) {
		case 1:
			ma_move_el (el, grade);
			if ((el = ma_el_to_repeat (MA_NOT_FORCED)) != NULL) {
				strcpy (q, el->q);
				strcpy (hint, "Press OK to see the answer");
			}
			else {
				state = 0;
				strcpy (hint, "Nothing more to repeat");
			}
			return;
		case 2:
			ma_move_el (el, grade);
			if (ma_get_new_el (MA_JUST_RETURN_IT, q, a) == 0) {
				a[0] = '\0';
				state = 1;
				strcpy (hint, "Press OK to see the answer.");
				return;
			}
			strcpy (hint, "Nothing more to learn.");
			return;
		case 3:
			/* not yet implemented */
			return;
		case 4:
			ma_move_el (el, grade);
			el = ma_el_to_repeat (MA_FORCED);
			assert (el);
			strcpy (q, el->q);
			strcpy (hint, "Press OK to see the answer");
			return;
		case 5:
			if ((el = ma_final_drill (GRADE_THRESHOLD))) {
				strcpy (q, el->q);
				strcpy (hint, "Press OK to see the answer");
			}
			else
				strcpy (hint, "Nothing more in Final drill queue");
			return;
		case 7:
			/* not implemented */
			return;
		case 8:
			/* not implemented */
		case 9:
		case 10:
			return;
		default:
			return;
	}
}

