/*

   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA

*/

/* for data.bin and user_data.bin
 *  - binary<->ASCII data converter
 * (This program converts a file with data for ANN)
 *
 *  - MemAid itself uses binary format (for faster access),
 *  but to see how the data look like, correct them or add more,
 *  you need this file in plain ASCII format) */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../core/memaid.h"

extern float f_interval (u_short);
extern u_short us_interval (float);
extern float f_n_of_rpt (u_short);
extern u_short us_n_of_rpt (float);
extern float f_grade (u_short);
extern u_short us_grade (float);

#define IN_OUT_N 5 /* neurons in input and output layer; number of data in each case */

void print_usage (char *s) {
	printf ("\t(NOT elements.bin data)\n");
	printf ("Usage:\n");
	printf ("%s 1 2 3\n", s);
	printf ("1 - name_of_file_to_convert_from\n");
	printf ("2 - name_of_file_to_create\n");
	printf ("3 - fo/fb (fa = from ascii, fb = from binary)\n");
	printf ("Examples:\n");
	printf ("%s data.bin data.txt fb\n", s);
	printf ("%s data.txt newdata.bin fa\n", s);
	return;
}

int from_ascii (FILE *we, FILE *wy) {
	float d[IN_OUT_N];
	u_short us[IN_OUT_N];
	puts ("from_ascii()");
	while (fscanf (we, "%hu,%hu,%hu,%hu,%hu", &us[0], &us[1], &us[2], &us[3], &us[4]) != EOF) {
	/*	printf ("%hu,%hu,%hu,%hu,%hu\n", us[0], us[1], us[2], us[3], us[4]); debug stuff */
		d[0] = f_interval (us[0]);
		d[1] = f_interval (us[1]);
		d[2] = f_n_of_rpt (us[2]);
		d[3] = f_grade    (us[3]);
		d[4] = f_interval (us[4]);
		fwrite (d, sizeof(float), IN_OUT_N, wy);
	}
	return 0;
}

					
int from_binary (FILE *we, FILE *wy) {
	float d[IN_OUT_N];

	puts ("from_binary()");
	while (fread (d, sizeof (float), IN_OUT_N, we) == IN_OUT_N) {
		fprintf (wy, "%hu,%hu,%hu,%hu,%hu\n", us_interval (d[0]), us_interval (d[1]),
			us_n_of_rpt (d[2]), us_grade (d[3]), us_interval (d[4]));
	}
	return 0;
}


int main (int argc, char *argv[]) {
	FILE *f_in, *f_out;
	int binary = 0;
	
	if (argc != IN_OUT_N - 1) {
		print_usage (argv[0]);
		exit (1);
	}
	
	switch (argv[3][1]) {
		case 'a':
			binary = 0;
			f_in = fopen (argv[1], "r");
			break;
		case 'b':
			binary = 1;
			f_in = fopen (argv[1], "rb");
			break;
		default:
			print_usage (argv[0]);
			exit (1);
	}

	if (!f_in) {
		printf ("Cannot open %s file\n", argv[1]);
		exit (1);
	}

	f_out = fopen (argv[2], "r");
	if (f_out) {
		printf ("%s already exist!\n", argv[2]);
		exit (2);
	}
	
	if (!binary) {
		f_out = fopen (argv[2], "wb");
		from_ascii (f_in, f_out);
	}
	else {
		f_out = fopen (argv[2], "w");
		from_binary (f_in, f_out);
	}
	
	fclose (f_in);
	fclose (f_out);
	return 0;
}

