/*

   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA

*/

/* Simple GUI for MemAid, based GraphApp.
 *
 * GraphApp www: http://www.cs.usyd.edu.au/~loki/graphapp/
 * MemAid   www: http://memaid.sf.net
 *
 * This interface looks very simple/primitive,
 * but has all main features and it's easy to implement next ones in it.
 *
 * It's actually very friendly to programmer:
 * easy to port (Windows and Unix supported by GraphApp),
 * easy to extend, easy to debug. Simple, small, fast.
 *
 * If you think about making yet another GUI (for Windows, Mac, whatever),
 * you may want to base your work on this file.
 * But beware: it was a quick hack.
 * I only coded it, not much tested. It may have a bugs.
 *
 * Enjoy,
 * David
***/


#include <stdio.h>
#include <stdlib.h>
#include "core/memaid.h"
#include <graphapp.h>

/* TODO:
	- statistics (a graphs)
	- resize of q_txt, a_txt and places of buttons and status on window_resize
	- support UTF-8 (isn't it working by default?), see how fonts are working
	- better list of elements:
		- basic data (number of reviews, scheduled time, etc.)
		- possibility to edit/delete/show choosen item
*/

#define GRADE_THRESHOLD 3

unsigned char state = 0;
/* 
	   0 - idle
	   1 - reviews
	   2 - new elements from queue
	   3 - new elements typed on-line
	   4 - forced ahead-of-time reviews
	   5 - final drill
	   6 - edit mode
	   7 - delete mode
*/

char ok_pressed = 0;
u_short hint = 0;
struct elem* el = NULL;
char* path_to_data_dir = NULL;
char q[MAX_Q_LEN], a[MAX_A_LEN];

App *app;
Control *q_txt, *a_txt, *okay_but, *stop_but, *hint_but;
Control *null_gr, *fail_gr, *poor_gr, *pass_gr, *good_gr, *ideal_gr;
Control *status;
Rect ok_pos;   /* default position of "OK" button, rest
		* of buttons are relative to "OK"
		*/
Rect win_pos;
Rect status_pos;

char *simple_stats (void) {
	static char scrap[256];
	int i, j;

	i = ma_list_n_of_els();
	j = ma_n_of_rpts_for_today();

	if (ma_iterate_list (NULL) == NULL) {
		strcpy (scrap, "Learning process hasn't started yet.");
		return scrap;
	}
	if (i > 1) {
		switch (j) {
		case 0:
			sprintf (scrap,
				"%d elements in the learning process. No repetitions left for today.", i);
			break;
		case 1:
			sprintf (scrap,
				"%d elements in the learning process. 1 repetition left for today.", i);
			break;
		default:
			sprintf (scrap,
				"%d elements in the learning process. %d repetitions left for today.", i, j);
			break;
		}
	}
	else { /* i == 1 */
		switch (j) {
		case 0:
			sprintf (scrap,
				"1 element in the learning process. No repetition left for today.");
			break;
		case 1:
			sprintf (scrap,
				"1 element in the learning process. 1 repetition left for today.");
			break;
		default:
			sprintf (scrap,
				"Oops. A weird error in ma_stats()");
			break;
		}
	}
	return scrap;
}

void reset_state (char *s) {
	state = hint = ok_pressed = 0;
	memset (q, 0, (size_t)MAX_Q_LEN);
	memset (a, 0, (size_t)MAX_A_LEN);
	app_set_control_text (q_txt, "");
	app_set_control_text (a_txt, "");
	app_set_control_text (status, s);
}

void greeting (void) {
	reset_state (simple_stats());
	return;
}

void update_el (struct elem *el) {
	int cur_l, old_l;

	cur_l = strlen (get_control_text (q_txt));
	old_l = strlen (el->q);
	
	if (cur_l > old_l)
		el->q = (char*) realloc (el->q, (cur_l+1) * sizeof(char));

	strcpy (el->q, get_control_text (q_txt));
	
	cur_l = strlen (get_control_text (a_txt));
	old_l = strlen (el->a);

	if (cur_l > old_l)
		el->a = (char*) realloc (el->a, (cur_l+1) * sizeof(char));
	
	strcpy (el->a, get_control_text (a_txt));
	return;
}
	
void on_exit (void) {
	puts ("Training ANN: wait a second ...");
	ma_train_ann_for_x_secs (1);
	ma_deinit();
	exit (0);
}

void set_status (char *s) {
	app_set_control_text (status, s);
}

void show_q  (const char *s) {
	char t[MAX_Q_LEN];
	strcpy (t, s);
	ma_remove_tags (t);
	app_set_control_text (q_txt, t);
	ma_run_tags (s, GUI_VERSION);
	app_set_control_text (a_txt, "");
}
void show_a (const char *s) {
	char t[MAX_A_LEN];
	strcpy (t, s);
	ma_remove_tags (t);
	app_set_control_text (a_txt, t);
	ma_run_tags (s, GUI_VERSION);
}

void on_gr (u_short grade) {
	if (ok_pressed == 0)
		return;

	ok_pressed = 0;

	switch (state) {
		case 1:
			update_el (el);
			printf ("New interval: %d\n", ma_move_el (el, grade));
			if ((el = ma_el_to_repeat (MA_NOT_FORCED)) != NULL) {
				show_q (el->q);
				set_status ("Press OK to see the answer.");
				return;
			}
			reset_state ("Nothing more to repeat. "
				"You can use \"Learn new items\" option from Learn menu");
			return;
		case 2:
			ma_get_new_el (MA_DELETE_IT, q, a);
			printf ("New interval: %d\n",
				ma_make_new_el (grade, get_control_text (q_txt), get_control_text (a_txt)));
			ma_store();
			if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
				show_q (q);
				set_status ("Press OK to see the answer.");
				return;
			}
			reset_state ("Nothing more to learn, but you can add new elements "
				"on your own by pressing \"Learn new items\" again.");
			return;
		case 3:
			printf ("New interval: %d\n",
				ma_make_new_el (grade, get_control_text (q_txt), get_control_text (a_txt)));
			ma_store();
			set_control_text (q_txt, "");
			set_control_text (a_txt, "");
			set_status ("Add next element, then press OK. Or press STOP.");
			return;
		case 4:
			update_el (el);
			printf ("New interval: %d\n", ma_move_el (el, grade));
			ma_store();
			el = ma_el_to_repeat (MA_FORCED);
			show_q (el->q);	
			set_status ("(Forced ahead-of-time repetitions mode!) "
					"Press \'OK\' to see the answer, \'STOP\' to stop.");
			return;
		case 5:
			update_el (el);
			if (grade >= GRADE_THRESHOLD) {
				printf ("New interval: %d\n", ma_move_el (el, grade));
				ma_store();
			}
			if ((el = ma_final_drill (GRADE_THRESHOLD))) {
				show_q (el->q);
				set_status ("Press OK to see the answer.");
				return;
			}
			reset_state ("Nothing more is in Final drill queue.");
			return;
		default:
			return;
	}
}

void on_ok (Control *c) {
	if (c != NULL)
		app_set_focus (c);
	if (ok_pressed == 1 && state != 6 && state != 7)
		return;
	switch (state) {
		case 0:
			return;
		case 1:
		case 4:
		case 5:
			show_a (el->a);
			set_status ("Grade?");
			break;
		case 2:
			show_a (a);
			set_status ("Grade?");
			break;
		case 3:	
			set_status ("Pick a grade: how well you remember the fact you just typed.");
			break;
		case 6:
			if (ok_pressed == 0) {
				el = ma_find_el (get_control_text(q_txt), get_control_text(a_txt));
				if (el == NULL) {
					reset_state ("Element not found.");
					return;
				}
				set_control_text (q_txt, el->q);
				set_control_text (a_txt, el->a);
				set_status ("Correct it and press OK.");
				break;
			}
			else {
				el->q = (char*) realloc (el->q,
						(strlen(get_control_text(q_txt))+1) * sizeof(char));
				el->a = (char*) realloc (el->a,
						(strlen(get_control_text(a_txt))+1) * sizeof(char));
				strcpy (el->q, get_control_text(q_txt));
				strcpy (el->a, get_control_text(a_txt));
				reset_state ("Element corrected.");
				return;
			}
		case 7:
			if (ok_pressed == 0) {
				el = ma_find_el (get_control_text(q_txt), get_control_text(a_txt));
				if (el == NULL) {
					reset_state ("Element not found.");
					return;
				}
				set_control_text (q_txt, el->q);
				set_control_text (a_txt, el->a);
				set_status ("Are you SURE you want to DELETE THIS element? "
					"Press \"OK\" button for \"YES\" or \"STOP\" for no.");
				break;
			}
			else {
				ma_delete_el (el);
				reset_state ("Element deleted.");
				return;
			}
		default:
			puts ("default in on_ok()");
			return;
	}
	ok_pressed = 1;
	hint = 0;
	return;
}
			
void on_gr_null  (Control *c) { on_gr (0); }
void on_gr_fail  (Control *c) { on_gr (1); }
void on_gr_poor  (Control *c) { on_gr (2); }
void on_gr_pass  (Control *c) { on_gr (3); }
void on_gr_good  (Control *c) { on_gr (4); }
void on_gr_ideal (Control *c) { on_gr (5); }
void quit    (MenuItem *m) { on_exit(); }
void on_stop (Control *c) { reset_state ("MemAid"); }

void on_export (MenuItem *m) {
	char *filename = NULL;

	if (ask_yes_no_cancel (app, "Confirm export", "Export elements to plain Q/A format?") == YES) {
		filename = ask_file_save (app, "Export", "Export", "export.txt");
		switch (ma_export (filename)) {
			case 0:
				ask_ok (app, "Export", "Elements exported.");
				break;
			case 1:
				ask_ok (app, "Export", "Nothing to export.");
				break;
			case 2:
				ask_ok (app, "Export", "File already exist.");
				break;
			case 3:
				ask_ok (app, "Export", "Cannot write/access the file.");
				break;
			default:
				ma_error ("on_export(): Cannot handle ma_export() result!\n");
				break;
		}
	}
	return;
}

char *dir (const char *s) {
	char *p, *last = NULL;
	static char buf[256];

	strncpy (buf, s, 256);
	p = buf;
	do {
		last = p;
#ifdef WIN32
		p = strchr (++p, '\\');
#else
		p = strchr (++p, '/');
#endif
	} while (p != NULL);
	if (last)
		*last = '\0';
	return buf;
}

void on_open_another (MenuItem *m) {
	char *filename = NULL;
	int i;
	
	filename = app_ask_file_open (app, "Open database",
			"Open", "elements.txt");
	if (filename == NULL)
		return;

	ma_deinit();
	i = ma_init (dir (filename));
	if (i & DIR_NOT_ACCESSIBLE) {
		ask_ok (app, "Invalid directory",
			"This directory doesn't contain valid MemAid database.");
		ma_init (path_to_data_dir);
	}
	else
		greeting();
	return;
}

extern struct elem *list;
#define MAX_DESC 45

void on_list (MenuItem *m) {
	Control *l;
	struct elem *el = list; 
	char **eld, *pos;
	int i, len;
	Window *w;

	if (el == NULL) {
		ask_ok (app, "Element list", "You don't have any elements in the learning process.");
		return;
	}
	
	w = new_window (app, rect (40,40,700,400), "ELements List", STANDARD_WINDOW);
	set_window_background (w, rgb (175,185,195));
		
	eld = (char**) malloc ((ma_list_n_of_els()+1) * sizeof(char*));
	i = 0;
	do {
		len = strlen (el->q);
		len = len > MAX_DESC ? MAX_DESC : len;
		eld[i] = malloc ((len+5) * sizeof(char));
		strncpy (eld[i], el->q, MAX_DESC - 1);
		eld[i][MAX_DESC-1] = '\0';
		pos = eld[i];
		while ((pos = strchr (pos, '\n'))) {
			*pos = '|';
			pos++;
		}
		strcat (eld[i], " ...");
		el = el->nxt;
		i++;
	} while (el != NULL);
	
	eld[i] = NULL;
	l = app_new_list_box (w, rect (10,10,650,380), eld, NULL);
	
	show_window (w);
	return;
}

void on_edit (MenuItem *m) {
	if (state != 0) {
		if (state != 6) /* already editing */
			return;
		ask_ok (app, "Edit", "If you want to edit an element, you have to first finish,"
				"or stop (by pressing STOP), current learning.");
	}
	set_control_text (q_txt, "");
	set_control_text (a_txt, "");
	set_status ("Type part of the question and/or the "
		"answer in q/a areas, and press \"OK\".");
	state = 6;
	return;
}

void on_delete (MenuItem *m) {
	if (state != 0) {
		if (state != 7) /* already deleting */
			return;
		ask_ok (app, "Edit", "If you want to delete an element, you have to first finish,"
			"or stop (by pressing STOP), current learning.");
	}
	set_control_text (q_txt, "");
	set_control_text (a_txt, "");
	set_status ("Type part of the question and/or the "
		"answer in q/a areas, and press \"OK\".");
	state = 7;
	return;
}

void on_delete_current (MenuItem *m) {
	switch (state) {
		case 0:
		case 3:
		case 6:
		case 7:
			return;
		default:
			break;
	}
	if (ask_yes_no_cancel (app, "Delete element",
		"Are you sure you want delete current element?") != YES)
		return;

	switch (state) {
		case 1:
			ma_delete_el(el);
			if ((el = ma_el_to_repeat (MA_NOT_FORCED)) != NULL) {
				show_q (el->q);
				ok_pressed = 0;
				set_status ("Press OK to see the answer.");
				break;
			}
			reset_state ("Nothing more to repeat. "
				"You can use \"Learn new items\" option from Learn menu");
			return;
		case 2:
			ma_get_new_el (MA_DELETE_IT, q, a);
			if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
				show_q (q);
				ok_pressed = 0;
				set_status ("Press OK to see the answer.");
				break;
			}
			reset_state ("Nothing more to learn, but you can add new elements "
				"on your own by pressing \"Learn new items\" again.");
			return;
		case 4:
			ma_delete_el (el);
			el = ma_el_to_repeat (MA_FORCED);
			show_q (el->q);
			ok_pressed = 0;
			set_status ("(Forced ahead-of-time repetitions mode!) "
				"Press \'OK\' to see the answer, \'STOP\' to stop.");
			return;
		case 5:
			ma_delete_el(el);
			if ((el = ma_final_drill (GRADE_THRESHOLD))) {
				show_q (el->q);
				ok_pressed = 0;
				set_status ("Press OK to see the answer.");
				return;
			}
			reset_state ("Nothing more in Final drill queue");
			return;
		default:
			ma_error ("on_delete_current(): Error: How I got here?\n");
	}
}

void on_hint (Control *c) {
	char tmp[MAX_A_LEN];
	unsigned int len, i;
	
	if (ok_pressed == 1 || state == 0 || hint >= 100)
		return;
	hint += 10;
	strcpy (tmp, el->a);
	ma_remove_tags (tmp);

	len = (unsigned int)strlen (tmp);
	for (i = 0; i < len; i++) {
		if (rand()%101 > hint)
			tmp[i] = ' ';
	}
	app_set_control_text (a_txt, tmp);
	if (hint == 100)
		ma_run_tags (el->a, GUI_VERSION);
	return;
}

void on_learn (MenuItem *m) {
	if (state != 0) {
		ask_ok(app, "Learn", "You are already in learning process!");
		return;
	}
	if ((el = ma_el_to_repeat (MA_NOT_FORCED)) != NULL) {
		show_q (el->q);
		state = 1;
		set_status ("Press OK to see the answer.");
		return;
	}
	reset_state ("Nothing more to learn. "
		"(You can use \"Learn new items\" option from Learn menu).");
	return;
}

void on_learn_new (MenuItem *m) {
	if (state != 0) {
		ask_ok (app, "Learn", "You are already in learning process!");
		return;
	}
	if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
		show_q (q);
		set_status ("Press OK");
		state = 2;
		return;
	}
	set_control_text (q_txt, "Type your question here");
	set_control_text (a_txt, "Type your answer here");
	set_status ("Type new item (question and answer), then press OK.");
	state = 3;
	return;
}	

void on_final_drill (MenuItem *m) {
	if (state != 0) {
		ask_ok (app, "Learn", "You are already in learning process!");
		return;
	}
	if ((el = ma_final_drill(GRADE_THRESHOLD))) {
		show_q (el->q);
		set_status ("Press OK to see the answer.");
		state = 5;
	}
	else
		reset_state ("Nothing is in Final drill queue.");
	return;
}

void on_forced_learn (MenuItem *m) {
	if (state != 0) {
		ask_ok (app, "Learn", "You are already in a learning process!");
		return;
	}
	if ((el = ma_el_to_repeat (MA_FORCED)) != NULL) {
		show_q (el->q);
		state = 4;
		set_status ("Press OK to see the answer.");
	}
	else
		reset_state ("Nothing to repeat - "
			"you don't have any elements in your learning process!");
	return;
}
	
void on_stats (MenuItem *m) {
	char s[4096];
	struct ma_stats_st fxs;
	ma_stats (&fxs);
	if (fxs.total_items == 0)
		sprintf (s, "No elements in the learning process.");
	else {
		sprintf (s, "First run: %s"
		"Items: %lu\n"
		"Days: %hu\n"
		"New items per day: %.2f\n\n"

		"Last max interval: by ANN %hu, real: %hu days\n"
		"Last min interval: by ANN %hu, real: %hu days\n"
		"Last mean interval: by ANN %hu, real: %hu days\n\n"

		"Current max/min/mean interval by ANN: %hu/%hu/%hu days\n\n"

		"Not counting reviews ahead of scheduled time:\n"
		"Sum of all made reviews: %lu\n"
		"Max number of reviews of an item so far: %hu\n"
		"Mean number of reviews: %hu\n\n"
	
		"Mean grade: %.2f\n"
		"Approximate retention: %.1f percent\n"
		"ANN TestError: %.3f\n"
		"Cases to train ANN: %lu (from user: %lu)\n",

		ctime(&(fxs.first_run)),
		fxs.total_items,
		fxs.days,
		fxs.item_per_day,

		fxs.max_l_ivl_ann, fxs.max_l_ivl_real,
		fxs.min_l_ivl_ann, fxs.min_l_ivl_real,
		fxs.mean_l_ivl_ann, fxs.mean_l_ivl_real,

		fxs.max_ivl,
		fxs.min_ivl,
		fxs.mean_ivl,

		fxs.sum_reviews,
		fxs.max_reviews,
		fxs.mean_reviews,
		fxs.mean_grade,

		fxs.retention,

		fxs.ann_error,
		fxs.ann_cases,
		fxs.user_ann_cases);
	}
	ask_ok (app, "Statistics", s);
	return;
}

void on_schedule (MenuItem *m) {
	char s[4096];
		/* TODO: graph */

	sprintf (s, "Items scheduled for review for next x day:\n"
		"0) today: %hu items\n"
		"1) tomorrow: %hu items\n"
		"2) %hu items\n"
		"3) %hu items\n"
		"4) %hu items\n"
		"5) %hu items\n"
		"6) %hu items\n"
		"7) %hu items\n"
		"8) %hu items\n"
		"9) %hu items\n"
		"10) %hu items\n"
		"11) %hu items\n"
		"12) %hu items\n"
		"13) %hu items\n"
		"14) %hu items\n"
		"Items scheduled for review in 15-60 days: %hu\n",
		ma_n_of_rpts_for_today(),
		ma_rpts_for (1,1),
		ma_rpts_for (2,2),
		ma_rpts_for (3,3),
		ma_rpts_for (4,4),
		ma_rpts_for (5,5),
		ma_rpts_for (6,6),
		ma_rpts_for (7,7),
		ma_rpts_for (8,8),
		ma_rpts_for (9,9),
		ma_rpts_for (10,10),
		ma_rpts_for (11,11),
		ma_rpts_for (12,12),
		ma_rpts_for (13,13),
		ma_rpts_for (14,14),
		ma_rpts_for (15,60));

	ask_ok (app, "Schedule", s);
	return;
}

void on_about (MenuItem *m) {
	Control *ab;
	Window *w = new_window (app, rect (40,40,700,400), "About MemAid", STANDARD_WINDOW);
	set_window_background (w, rgb (175,185,195));
	
	ab = app_new_label (w, rect (10,10,600,350),
		"MemAid " MEMAID_VERSION "\nCopyright (c) 2003 David Calinski, Karsten Kutza\n\n"
		MEMAID_DESCRIPTION
		"\n\nhttp://memaid.sf.net/\n\n"
		MEMAID_CORE_THANK_AND_CREDIT_LIST, ALIGN_LEFT + VALIGN_CENTER);
	show_window (w);
	return;
}

void on_license (MenuItem *m) {
	Control *lic;
	Window *w = new_window (app, rect (40,40,750,460), "MemAid License", STANDARD_WINDOW);
	lic = app_new_label (w, rect (10,10,740,420), MEMAID_LICENSE, ALIGN_LEFT + VALIGN_CENTER);
	set_window_background (w, rgb (175,185,195));
	show_window (w);
	return;
}

void on_keyw (Window *w, unsigned long key) {
	switch (key) {
		case ' ':
		case '\n':
			on_ok(NULL);
			printf ("OK\n");
			return;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
			on_gr (key - '0');
			printf ("on_gr(%lu)\n", key - '0');
			return;
		default:
			return;
	}
}

void on_key (Control *c, unsigned long key) {
	switch (key) {
		case ' ':
		case '\n':
			on_ok(NULL);
			printf ("OK\n");
			return;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
			on_gr (key - '0');
			printf ("on_gr(%lu)\n", key - '0');
			return;
		default:
			return;
	}
}

int main (int argc, char *argv[]) {
	Window *win;
	MenuBar *mb;
	Menu *m[6];
	MenuItem *mi[20];
	
	ok_pos = rect (250,490,60,30);
	win_pos  = rect (30,30,730,600);
	status_pos = rect (30,590,700,30);
	
	int i = 0;
	char *path_to_data_dir = NULL;

	while (++i < argc) {
		if (argv[i][0] == 'd')
			path_to_data_dir = &argv[i][1];
	}
	if (path_to_data_dir == NULL)
		path_to_data_dir = "data/";

	i = ma_init (path_to_data_dir);
	
	if (i & DIR_NOT_ACCESSIBLE || i & MEMAID_ALREADY_RUNNING)
		return 1;
	
	app   = new_app (argc, argv);
	win   = new_window (app, win_pos, "MemAid", STANDARD_WINDOW);
	set_window_background (win, rgb (175,185,195));
	
	mb    = new_menu_bar (win);
	m[0]  = new_menu (mb, "File");
	mi[0] = new_menu_item (m[0], "Open another", 0, on_open_another);

	mi[1] = new_menu_item (m[0], "List",   0, on_list);
	mi[2] = new_menu_item (m[0], "Export", 0, on_export);
	mi[3] = new_menu_item (m[0], "Quit",  'Q', quit);

	m[1]  = new_menu (mb, "Edit");
	mi[4] = new_menu_item (m[1], "Edit an item",     0, on_edit);
	mi[5] = new_menu_item (m[1], "Delete an item",   0, on_delete);
	mi[6] = new_menu_item (m[1], "Delete current item",  0, on_delete_current);
	
	m[2]   = new_menu (mb, "Learn");
	mi[7]  = new_menu_item (m[2], "Learn", 0, on_learn);
	mi[8]  = new_menu_item (m[2], "Learn new items", 0, on_learn_new);
	mi[9]  = new_menu_item (m[2], "Final Drill", 0, on_final_drill);
	mi[10] = new_menu_item (m[2], "Force ahead of scheduled time repetitions", 0, on_forced_learn);

	m[3]   = new_menu (mb, "Statistics");
	mi[11] = new_menu_item (m[3], "Statistics", 0, on_stats);
	mi[12] = new_menu_item (m[3], "Schedule",   0, on_schedule);

	m[4]   = new_menu (mb, "About");
	mi[13] = new_menu_item (m[4], "License", 0, on_license);
	mi[14] = new_menu_item (m[4], "About",   0, on_about);

	q_txt  = new_text_box (win, rect (60, 60,600,200), "Question");
	a_txt  = new_text_box (win, rect (60,280,600,200), "Answer");

	okay_but = new_button (win, ok_pos, "Okay", on_ok);
	ok_pos.x += 65;
	hint_but = new_button (win, ok_pos, "Hint", on_hint);
	ok_pos.x += 65;
	stop_but = new_button (win, ok_pos, "Stop", on_stop);
	
	ok_pos.x -= 230;
	ok_pos.y += 35;
	null_gr  = new_button (win, ok_pos, "Null",   on_gr_null);
	ok_pos.x += 65;
	fail_gr  = new_button (win, ok_pos, "Fail",   on_gr_fail);
	ok_pos.x += 65;
	poor_gr  = new_button (win, ok_pos, "Poor",   on_gr_poor);
	ok_pos.x += 65;
	pass_gr  = new_button (win, ok_pos, "Pass",   on_gr_pass);
	ok_pos.x += 65;
	good_gr  = new_button (win, ok_pos, "Good",   on_gr_good);
	ok_pos.x += 65;
	ideal_gr = new_button (win, ok_pos, "Ideal", on_gr_ideal);

	status   = new_label  (win, status_pos, "Idle.", ALIGN_CENTER);

	app_on_window_key_down  (win, on_keyw);
	app_on_control_key_down (okay_but, on_key);
	
	show_window (win);
	greeting ();
	main_loop (app);
	
	on_exit();
	return 0;
}

