/*

   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA

*/
 
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>

#define MAX_LINE 256


static void panic (char *s) {
	fputs (s, stderr);
	exit (1);
	return;
}


static void print_usage (char *s) {
	printf ("This program makes a \"queue.txt\" (q&a format readable by MemAid)\n");
	printf ("from SuperMemo7 q&a format\n");
	printf ("Usage:\n");
	printf ("%s 1 2\n", s);
	printf ("1 - input file\n");
	printf ("2 - output file (e.g. queue.txt)\n");
	return;
}


int main (int argc, char *argv[]) {
	FILE *f_in, *f_out;
	char buf[MAX_LINE], *pos;
	int state;

	if (argc < 3) {
		print_usage (argv[0]);
		exit (1);
	}

	f_in = fopen (argv[1], "r");
	if (!f_in) {
		printf ("Cannot open %s\n", argv[1]);
		exit (2);
	}

	f_out = fopen (argv[2], "r");
	if (f_out) {
		printf ("File \"%s\" already exist!\n", argv[2]);
		exit (3);
	}

	f_out = fopen (argv[2], "w");
	if (!f_out) {
		printf ("Cannot open file \"%s\" for writing!\n", argv[2]);
		exit (4);
	}


	state = 0;
	
	/* state:
	   0 - read nothing, or empty line (we now will search for a Q)
	   	no tags opened
		
	   1 - recently read Q             (we now will search for a next Q or A)
	   	<q>We are here...
		
	   2 - recently read A             (we now will search for a next A or empty line)
	   	<a>We are here...
	*/
	
	while (fgets (buf, MAX_LINE, f_in)) {
		if ((pos=strchr (buf, '\n')))
				*pos = '\0';
				
		switch (state) {
			case 0:
				if (buf[0] == 'Q') {
					state = 1;
					fprintf (f_out, "<q>%s", &buf[3]);
				}
				break;
			case 1:
				if (buf[0] == 'Q') {
					fprintf (f_out, "\n%s", &buf[3]);
				}
				else {
					if (buf[0] != 'A')
						panic ("Error. No \"A:\" just after \"Q:\"\n");
					fprintf (f_out, "</q>\n<a>%s", &buf[3]);
					state = 2;
				}
				break;
			case 2:
				if (buf[0] == 'A') {
					fprintf (f_out, "\n%s", &buf[3]);
				}
				else {
					if (buf[0] == 'Q')
						panic ("Error. \"Q:\" just after an \"A:\".\n");
					fprintf (f_out, "</a>\n\n");
					state = 0;
				}
				break;
			}
	}
	switch (state) {
		case 0:
			break;
		case 1:
			panic ("Error. Lack of \"A:\".\n");
		case 2:
			fprintf (f_out, "</a>\n");
	}
	
	fclose (f_in);
	fclose (f_out);
	return 0;
}
