/*
   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA
*/

#include "fox.h"

char state = 0;
/* 0 - idle
   1 - repetitions
   2 - new elements from queue
   3 - new elements typed on-line
   4 - forced repetitions
   5 - final drill
   6 - edit mode
   7 - delete mode
*/

static char *simple_stats (void) {
	static char scrap[256];
	int i, j, k;

	i = ma_list_n_of_els();
	j = ma_n_of_rpts_for_today();
	k = ma_rpts_drill(GRADE_THRESHOLD);

	if (ma_iterate_list (NULL) == NULL) {
		strcpy (scrap, "Learning process hasn't started yet.");
		return scrap;
	}
	if (i > 1) {
		switch (j) {
		case 0:
			sprintf (scrap,
				"%d elements. No repetitions left for today.", i);
			break;
		case 1:
			sprintf (scrap,
				"%d elements. 1 repetition left for today.", i);
			break;
		default:
			sprintf (scrap,
				"%d elements. %d repetitions left for today.", i, j);
			break;
		}
	}
	else { /* i == 1 */
		switch (j) {
		case 0:
			sprintf (scrap,
				"1 element. No repetition left for today.");
			break;
		case 1:
			sprintf (scrap,
				"1 element. 1 repetition left for today.");
			break;
		default:
			sprintf (scrap,
				"Oops. A weird error in simple_stats()");
			break;
		}
	}
	return scrap;
}

void ma_window::greeting (void) {
	switch (rand()%3) {
		case 0:
			reset_state (GREETING_TXT MEMAID_QUOTE1, NULL, simple_stats());
			break;
		case 1:
			reset_state (GREETING_TXT MEMAID_QUOTE2, NULL, simple_stats());
			break;
		case 2:
			reset_state (GREETING_TXT MEMAID_QUOTE3, NULL, simple_stats());
			break;
	}
}

FXDEFMAP(ma_window) ma_window_map[]={
	//________Message_Type_____________________ID_______________Message_Handler_______
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_SHOW_ABOUT,	ma_window::show_about),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_SHOW_LICENSE,	ma_window::show_license),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_HELP,		ma_window::show_help),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_TIPS,		ma_window::show_tips),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_OPEN,		ma_window::on_open_another),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_LIST,		ma_window::on_list),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_LEARN,		ma_window::on_learn),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_LEARN_NEW,	ma_window::on_learn_new),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_LEARN_NFQ,	ma_window::on_learn_nfq),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_FORCE_RPT,	ma_window::on_force_rpt),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_FINAL_DRILL,	ma_window::on_final_drill),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_EXPORT,		ma_window::on_export),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_EDIT_IT,		ma_window::on_edit),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_EDIT_CURRENT,	ma_window::on_edit_current),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_DELETE,		ma_window::on_delete),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_DELETE_CURRENT,	ma_window::on_delete_current),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_STOP,		ma_window::on_stop),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_OK,		ma_window::on_ok_button),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_HINT,		ma_window::on_hint),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_STATS,		ma_window::on_stats),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_SHOW_SCHEDULE,	ma_window::on_show_schedule),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_GRADE_0,		ma_window::grade_0),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_GRADE_1,		ma_window::grade_1),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_GRADE_2,		ma_window::grade_2),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_GRADE_3,		ma_window::grade_3),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_GRADE_4,		ma_window::grade_4),
	FXMAPFUNC(SEL_COMMAND,		ma_window::ID_GRADE_5,		ma_window::grade_5),
	FXMAPFUNC(SEL_TIMEOUT,		ma_window::ID_TRAIN_ANN,	ma_window::on_train_ann),
	FXMAPFUNC(SEL_KEYPRESS,		ma_window::ID_KEY_PRESS,	ma_window::on_key_press)
};

FXIMPLEMENT(ma_window,FXMainWindow,ma_window_map,ARRAYNUMBER(ma_window_map))

	
ma_window::ma_window (FXApp *ap) : FXMainWindow (ap, "MemAid", NULL, NULL, DECOR_ALL, 60, 60, 700, 500) {
	menu_bar   = new FXMenubar  (this);
	file_menu  = new FXMenuPane (this);
	edit_menu  = new FXMenuPane (this);
	learn_menu = new FXMenuPane (this);
	stats_menu = new FXMenuPane (this);
	about_menu = new FXMenuPane (this);

	new FXMenuTitle (menu_bar, "&File",       NULL,  file_menu);
	new FXMenuTitle (menu_bar, "&Edit",       NULL,  edit_menu);
	new FXMenuTitle (menu_bar, "&Learn",      NULL, learn_menu);
	new FXMenuTitle (menu_bar, "&Statistics", NULL, stats_menu);
	new FXMenuTitle (menu_bar, "&About",      NULL, about_menu);

	new FXMenuCommand (file_menu, "Open another\t\tOpens another database",
			NULL, this, ma_window::ID_OPEN);

	new FXMenuCommand (file_menu, "List\t\tShows all elements",
			NULL, this, ma_window::ID_LIST);

	new FXMenuCommand (file_menu, "Export\t\tExport questions and answers to a file",
			NULL, this, ma_window::ID_EXPORT);

	new FXMenuCommand (file_menu, "Exit\t\tExit",
			NULL, getApp(), FXApp::ID_QUIT);

	new FXMenuCommand (edit_menu, "&Edit this item\tCtrl-E\t"\
			"Allows you to correct currently visible element",
			NULL, this, ma_window::ID_EDIT_CURRENT);

	new FXMenuCommand (edit_menu, "Edit an item\t\tShows editing dialog",
			NULL, this, ma_window::ID_EDIT_IT);

	new FXMenuCommand (edit_menu, "&Delete this item\tCtrl-D\t"
			"Delete current visible item from the learning process",
			NULL, this, ma_window::ID_DELETE_CURRENT);

	new FXMenuCommand (edit_menu, "Delete an item\t\tShows deleting dialog",
			NULL, this, ma_window::ID_DELETE);

	new FXMenuCommand (learn_menu, "&Learn\tCtrl-L\tExecutes repetitions",
			NULL, this, ma_window::ID_LEARN);

	new FXMenuCommand (learn_menu, "Learn new items\t\tLearn new items from prepared queue.txt file",
			NULL, this, ma_window::ID_LEARN_NFQ);

	new FXMenuCommand (learn_menu, "Input new items\t\tAdds new items to the learning process",
			NULL, this, ma_window::ID_LEARN_NEW);
	
	new FXMenuCommand (learn_menu, "Final drill\t\t"
		"Process again items that had review today, but got a bad grade",
		NULL, this, ma_window::ID_FINAL_DRILL);

	new FXMenuSeparator (learn_menu);
		
	new FXMenuCommand (learn_menu, "Force repetitions\t\t"
			"Forces executes repetitions even out of schedule (not optimal!)",
			NULL, this, ma_window::ID_FORCE_RPT);

	new FXMenuCommand (stats_menu, "Basic statistics\t\tShows some statistics about your learning process",
			NULL, this, ma_window::ID_STATS);

	new FXMenuCommand (stats_menu, "Schedule\t\tShows how many elements are sheduled for next days", 
			NULL, this, ma_window::ID_SHOW_SCHEDULE);
		
	new FXMenuCommand (about_menu, "Help\t\tBasic help",
			NULL, this, ma_window::ID_HELP);
	
	new FXMenuCommand (about_menu, "Tips\t\tUseful hints and tips",
			NULL, this, ma_window::ID_TIPS);

	new FXMenuCommand (about_menu, "License\t\tTerms of use",
			NULL, this, ma_window::ID_SHOW_LICENSE);

	new FXMenuCommand (about_menu, "About\t\tShort info about MemAid", 
			NULL, this, ma_window::ID_SHOW_ABOUT);

	contents = new FXHorizontalFrame (this, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y,
		0,0,0,0,0,0,0,0);

	qa_frame = new FXVerticalFrame (contents,
		FRAME_SUNKEN | LAYOUT_FILL_X |LAYOUT_FILL_Y| LAYOUT_TOP| LAYOUT_LEFT,
		0,0,0,0,10,10,10,10);
	
	qtxt = new FXText (qa_frame, this, ID_KEY_PRESS, FRAME_SUNKEN | FRAME_THICK | 
			TEXT_WORDWRAP | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT);

	status_bar  = new FXStatusbar  (qa_frame, LAYOUT_BOTTOM | LAYOUT_FILL_X);
	status_line = status_bar->getStatusline();

	atxt = new FXText (qa_frame, this, ID_KEY_PRESS, FRAME_SUNKEN | FRAME_THICK |
			TEXT_WORDWRAP | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_BOTTOM | LAYOUT_LEFT);

	button_frame = new FXVerticalFrame (contents,
		FRAME_SUNKEN | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);

	new FXButton (button_frame, "Stop", NULL, this, ID_STOP,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);
	new FXButton (button_frame, "Hint", NULL, this, ID_HINT,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);
	new FXButton (button_frame, "OK", NULL,   this, ID_OK,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);

  	new FXHorizontalSeparator (button_frame , SEPARATOR_RIDGE|LAYOUT_FILL_X);

	new FXButton (button_frame, "0\t\tGrade \"Null\": you don't remember it", NULL,
			this, ID_GRADE_0,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,7,7);
	new FXButton (button_frame, "1\t\tGrade \"Fail\": you failed to recollect it, but was not so far",
			NULL, this, ID_GRADE_1,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,7,7);
	new FXButton (button_frame, "2\t\tGrade \"Poor\": you had serious problems with recollecting, "\
			"but finally bring it back",
			NULL, this, ID_GRADE_2,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,7,7);
	new FXButton (button_frame, "3\t\tGrade: \"Pass\": you had problems with recollecting it, "\
			"but finally you bring it back",
			NULL,     this, ID_GRADE_3,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,7,7);
	new FXButton (button_frame, "4\t\tGrade \"Good\": you remember it",
			NULL, this, ID_GRADE_4,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,7,7);
	new FXButton (button_frame, "5\t\tGrade \"Perfect\": you remember it very well",
			NULL, this, ID_GRADE_5,
		FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,7,7);
	greeting();
}

void ma_window::reset_state (char* qt, char* at, char* st) {
	state = hint = ok_pressed = 0;
        int i;
        i = ma_n_of_rpts_for_today();
	if (qt == NULL)
		qtxt->setText ("");
	else
		qtxt->setText (qt);
	if (at == NULL)
		atxt->setText ("");
	else
		atxt->setText (at);
	memset (q, 0, (size_t)MAX_Q_LEN);
	memset (a, 0, (size_t)MAX_A_LEN);
	if (st == NULL)
                
		set_status ("idle");
	else
		set_status (st);
	atxt->setEditable (false);
	qtxt->setEditable (false);
	edit_current = false;
}

void ma_window::add_to_status (char *s) {
	FXString str_to_add (s);
	str_to_add.append ("\n |"); // according to FOX docs, it should highlight everything up to '\n'
	FXString old_status (status_line->getNormalText()); 
	str_to_add.append (old_status);
	status_line->setNormalText (str_to_add);
}

long ma_window::show_about (FXObject*, FXSelector, void*) {
	FXMessageBox::information (this, MBOX_OK, "About MemAid",
		"MemAid " MEMAID_VERSION "\nCopyright (c) 2003 David Calinski, Karsten Kutza\n\n"\
		MEMAID_DESCRIPTION
		"\n\nhttp://memaid.sf.net/\n\n"\
		"This software uses the FOX Toolkit Library (http://www.fox-toolkit.org).\n\n\n"\
		MEMAID_CORE_THANK_AND_CREDIT_LIST);
	return 1;
}

long ma_window::show_help (FXObject*, FXSelector, void*) {
	FXMessageBox::information (this, MBOX_OK, "Basic Help", MEMAID_MANUAL);
	return 1;
}

long ma_window::show_tips (FXObject*, FXSelector, void*) {
	FXMessageBox::information (this, MBOX_OK, "Tips", MEMAID_TIPS);
	return 1;
}

long ma_window::show_license (FXObject*, FXSelector, void*) {
	FXMessageBox::information (this, MBOX_OK,"License",  MEMAID_LICENSE);
	return 1;
}
  
long ma_window::on_ok_button (FXObject*, FXSelector, void*) {
	on_ok();
	return 1;
}

void ma_window::on_ok (void) {
	if (ok_pressed == 1 && state != 6 && state != 7)
		return;

	FXString qs (qtxt->getText());
	FXString as (atxt->getText());

	switch (state) {
		case 1:
		case 4:
		case 5:
			show_a (el->a);
			set_status ("Your grade?");
			break;
		case 2:
			show_a (a);
			set_status ("Your grade?");
			break;
		case 3:
			set_status ("Pick a grade: how well you remember the fact just typed.");
			break;
		case 6:
			if (ok_pressed == 0) {
				el = ma_find_el (qs.text(), as.text());
				if (el == NULL) {
					reset_state ("Element not found");
					return;
				}
				qtxt->setText (el->q);
				atxt->setText (el->a);
				set_status ("Please correct question and/or answer, "
					"then press \"OK\" button.");
				break;
			}
			else {
				el->q = (char*) realloc (el->q, (strlen(qs.text())+1) * sizeof(char));
				el->a = (char*) realloc (el->a, (strlen(as.text())+1) * sizeof(char));
				strcpy (el->q, qs.text());
				strcpy (el->a, as.text());
				reset_state ("Element corrected.");
				return;
			}
		case 7:
			if (ok_pressed == 0) {
				el = ma_find_el (qs.text(), as.text());
				if (el == NULL) {
					reset_state ("Element not found.");
					return;
				}
				qtxt->setText (el->q);
				atxt->setText (el->a);
				atxt->setEditable (false);
				qtxt->setEditable (false);
				set_status ("Are you *sure* you want to DELETE this element? "\
					"Press \"OK\" button for \"YES\" or \"STOP\" for no.");
				break;
			}
			else {
				ma_delete_el (el);
				reset_state ("Element deleted.");
				return;
			}
		default:
			return;
	}
	ok_pressed = 1;
	hint = 0;
}

long ma_window::on_stop (FXObject*, FXSelector, void*) {
	reset_state ("MemAid");
	return 1;
}

long ma_window::on_export(FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::warning (this, MBOX_OK, "Export", 
			"You are in a learning process! Before you can export elements,\n"\
			"you have to finish, or stop (by pressing STOP), current learning.");
		return 1;
	}
	FXString filename;
	FXFileDialog expdialog (this, "Export items");
	expdialog.setPattern ("*.txt");
	filename = path_to_data_dir;
	filename.append ("export.txt");
	expdialog.setFilename (filename);
	if (!expdialog.execute())
		return 1;
	filename = expdialog.getFilename();
	
	switch (ma_export(filename.text())) {
		case 0:
			reset_state ("I just have exported questions and answers to export.txt file\n"
				"in your MemAid data directory (e.g. $HOME/.memaid/)");
			break;
		case 1:
			set_status ("Nothing to export.");
			break;
		case 2:
			reset_state ("Not done.\n"
				"First delete current export.txt file from your data directory!\n"\
				"(e.g. $HOME/.memaid/export.txt)");
			break;
		default:
			ma_error ("Cannot handle ma_export reply in ma_window::on_export()\n");
			break;
	}
	return 1;
}

void ma_window::after_grade (u_short grade) {
	if (ok_pressed == 0 && state != 3)
		return;
	ok_pressed = 0;
	
	u_short i; int j,k;
        j = ma_n_of_rpts_for_today(); /*bruno*/
        k = ma_rpts_drill(GRADE_THRESHOLD); /*bruno*/
	switch (state) {
		case 1: 
			update_current (el);
			i = ma_move_el (el, grade);
			ma_store();
			printf ("%d-%d New interval = %d\n",j,k, i); // what about printing it out somewhere in GUI?
			if ((el = ma_el_to_repeat ()) != NULL) {
				show_q (el->q);
				set_status ("Press OK to see the answer.");
				timerhandle = application->addTimeout (100,main_win,ID_TRAIN_ANN);
				return;
			}
			reset_state ("Nothing more to repeat. "\
				"You can use \"Learn new items\" option from Learn menu");
			timerhandle = application->addTimeout (100,main_win,ID_TRAIN_ANN);
			return;
		case 2:
			ma_get_new_el (MA_DELETE_IT, q, a); // deletes learnt element from queue.txt
			i = ma_make_new_el (grade, qtxt->getText().text(), atxt->getText().text());
			ma_store();
			printf ("%d-%d New interval = %d\n",j, k, i);
			if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
				show_q (q);
				set_status ("Press OK to see the answer.");
				return;
			}
			reset_state ("Nothing more to learn, but you can add new elements "
				"on your own by pressing \"Learn new items\" again.");
			return;
		case 3:
			i = ma_make_new_el (grade, qtxt->getText().text(), atxt->getText().text());
			ma_store();
			printf ("%d-%d New interval = %d\n", j, k, i);
			set_status ("Add a next element, then press a grade; or press STOP to stop.");
			qtxt->setText ("");
			atxt->setText ("");
			return;
		case 4:
			update_current(el);
			i = ma_move_el (el, grade);
			ma_store();
			printf ("%d-%d New interval = %d\n",j, k, i); // what about printing it out somewhere in GUI?
			el = ma_el_force_repeat ();
			show_q (el->q);
			set_status ("(Not optimal learning! Forced repetitions mode.) "\
					"Press \'OK\' to see the answer, \'STOP\' to stop.");
			timerhandle = application->addTimeout (100, main_win, ID_TRAIN_ANN);
			return;
		case 5:
			update_current(el);
			if (grade >= GRADE_THRESHOLD) {
				i = ma_move_el (el, grade);
				ma_store();
				printf ("%d-%d New interval = %d\n",j, k, i);
				ma_store();
			}
			if ((el = ma_final_drill (GRADE_THRESHOLD))) {
				show_q (el->q);
				set_status ("Press OK to see the answer");
				timerhandle = application->addTimeout (100, main_win, ID_TRAIN_ANN);
				return;
			}
			reset_state ("Nothing more in Final drill queue");
			timerhandle = application->addTimeout (100, main_win, ID_TRAIN_ANN);
			return;
		default:
			puts ("Default in after_grade()");
	}
	return;
}

int ma_window::key_hint (long key) {
	if (ok_pressed == 1 || state == 0 || state == 3 || state == 6 || state == 7)
		return 0;

	u_short alen = (u_short) strlen (((state==2) ? a : el->a));
	
	char tmp[MAX_A_LEN];
	strncpy (tmp, ((state==2) ? a : el->a), hint + 1);
	
	if (key == tmp[hint]) { // this is not very right: 'key' don't have to be ASCII
		tmp[++hint] = '\0';
		ma_remove_tags (tmp);
		if (hint == alen) {
			atxt->setText (tmp);
			ma_run_tags (((state==2) ? a : el->a), GUI_VERSION);
			ok_pressed = 1;
			hint = 0;
			set_status ("Your grade?");
			return 1;
		}
		atxt->setText (tmp);
		return 1;
	}
	return 0;
}

long ma_window::on_hint (FXObject*, FXSelector, void*) {
	if (ok_pressed == 1 || state == 0 || state == 3 || state == 6 || state == 7)
		return 1;
	
	u_short alen = (u_short) strlen (((state==2) ? a : el->a));
	
	if (hint >= alen)
		return 1;
	
	++hint;
	
	char tmp[MAX_A_LEN];
	assert (MAX_A_LEN > alen);
	strncpy (tmp, ((state==2) ? a : el->a), hint);
	tmp[hint] = '\0';
	ma_remove_tags (tmp);
	if (hint == alen) {
		atxt->setText (tmp);
		ma_run_tags (((state==2) ? a : el->a), GUI_VERSION);
		ok_pressed = 1;
		hint = 0;
		set_status ("Your grade?");
		return 1;
	}
	atxt->setText (tmp);
	return 1;
}

long ma_window::on_learn (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::warning (this, MBOX_OK, "Learn", 
			"You are already in a learning process!");
		return 1;
	}
	if ((el = ma_el_to_repeat ()) != NULL) {
		show_q (el->q);
		state = 1;
		set_status ("Press OK to see the answer.");
		return 1;
	}
	reset_state ("Nothing more to learn. (You can use \"Learn new items\" option from Learn menu).");
	return 1;
}

long ma_window::on_learn_nfq (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::warning (this, MBOX_OK, "Learn", 
			"You are in a learning process! Before you can learn new elements,\n"\
			"you have to finish, or stop (by pressing STOP), current learning.");
		return 1;
	}
	if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
		show_q (q);
		set_status ("Press OK");
		state = 2;
		return 1;
	}
	FXMessageBox::information (this, MBOX_OK, "Learn new", 
			"No items in queue.txt file, you can input new items on-line.");
	qtxt->setText  ("Type your question here");
	atxt->setText  ("Type your answer here");
	set_status ("Type a new item (question, answer), "
			" then press a grade - how well you remember it.");
	atxt->setEditable (true);
	qtxt->setEditable (true);
	state = 3;
	return 1;
}

long ma_window::on_learn_new (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::warning (this, MBOX_OK, "Learn", 
			"You are in a learning process! Before you can learn new elements,\n"\
			"you have to finish, or stop (by pressing STOP), current learning.");
		return 1;
	}
	qtxt->setText  ("Type your question here");
	atxt->setText  ("Type your answer here");
	set_status ("Type a new item (question, answer), "
			"then press a grade - how well you remember it.");
	atxt->setEditable (true);
	qtxt->setEditable (true);
	state = 3;
	return 1;
}

long ma_window::on_force_rpt (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::warning (this, MBOX_OK, "Learn",
			"You already are in a learning process!\n");
		return 1;
	}
	FXMessageBox::warning (this, MBOX_OK, "Forced repetitions",
		"You have just turned on \"forced repetitions\" mode.\n"
		"It will keep executing reviews of all items (even if they are not scheduled for a review yet),\n"
		"until you press \"STOP\" button.\n"
		"This is not optimal learning.\n");
	
	if ((el = ma_el_force_repeat ()) != NULL) {
		show_q (el->q);
		state = 4;
		set_status ("Press OK to see the answer.");
	}
	else
		reset_state ("Nothing to forced repeat - "
				"you don't have any elements in the learning process!");
	return 1;
}


long ma_window::on_final_drill (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::warning (this, MBOX_OK, "Learn", 
			"You are in a learning process! Before you can go into Final drill,\n"\
			"you have to finish, or stop (by pressing STOP), current learning.");
		return 1;
	}
	if ((el = ma_final_drill(GRADE_THRESHOLD))) {
		show_q (el->q);
		set_status ("Press OK to see the answer");
		state = 5;
	}
	else
		reset_state ("Nothing is in Final drill queue.");
	return 1;
}

long ma_window::on_edit (FXObject*, FXSelector, void*) {
	if (state != 0) {
		if (state == 6)
			return 1; // already in edit mode
		FXMessageBox::information (this, MBOX_OK, "Edit", 
			"If you want to edit an element, you have to first finish, "\
			"or stop (by pressing STOP), current learning.");
		return 1;
	}
	qtxt->setText ("");
	atxt->setText ("");
	set_status ("Type part of the question and/or the "
			"answer in q/a areas, and press \"OK\".");
	atxt->setEditable (true);
	qtxt->setEditable (true);
	state = 6;
	return 1; /* FIXME: <exec> tags dissapears */
}

long ma_window::on_edit_current (FXObject*, FXSelector, void*) {
	switch (state) {
		case 2:
			show_q(q, true);
			show_a(a, true);
			break;
		case 1:
		case 4:
		case 5:
			show_q(el->q, true);
			show_a(el->a, true);
			break;
		default:
			FXMessageBox::information (this, MBOX_OK, "Edit current", 
				"Cannot edit current element in actual learning state");
			return 1;
	}
	atxt->setEditable (true);
	qtxt->setEditable (true);
	edit_current = true;
	return 1; /* FIXME: <exec> tags dissapears */
}

long ma_window::on_delete_current (FXObject*, FXSelector, void*) {
	switch (state) {
		case 1:
			if (MBOX_CLICKED_NO == FXMessageBox::question (this, MBOX_YES_NO, "Delete current",
				"Are you sure you want delete current element?"))
				return 1;
			ma_delete_el (el);
			if ((el = ma_el_to_repeat ()) != NULL) {
				show_q (el->q);
				ok_pressed = 0;
				set_status ("Press OK to see the answer.");
				break;
			}
			reset_state ("Nothing more to repeat. "\
					"You can use \"Learn new items\" option from Learn menu.");
			break;
		case 2:
			if (MBOX_CLICKED_NO == FXMessageBox::question (this, MBOX_YES_NO, "Delete current",
				"Are you sure you want delete current element??"))
				return 1;
			ma_get_new_el (MA_DELETE_IT, q, a);
			if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
				show_q (q);
				ok_pressed = 0;
				set_status ("Press OK to see the answer.");
				break;
			}
			reset_state ("Nothing more to learn, but you can add new elements "
				"on your own by pressing \"Learn new items\" again.");
			break;
		case 4:
			if (MBOX_CLICKED_NO == FXMessageBox::question (this, MBOX_YES_NO, "Delete current",
				"Are you sure you want delete current element?"))
				return 1;
			ma_delete_el (el);
			el = ma_el_force_repeat ();
			show_q (el->q);
			ok_pressed = 0;
			set_status ("(Forced ahead-of-time repetitions mode!) "
				"Press \'OK\' to see the answer, \'STOP\' to stop.");
			break;

		case 5:
			if (MBOX_CLICKED_NO == FXMessageBox::question (this, MBOX_YES_NO, "Delete current",
				"Are you sure you want delete current element??"))
				return 1;
			ma_delete_el (el);
			if ((el = ma_final_drill (GRADE_THRESHOLD))) {
				show_q (el->q);
				ok_pressed = 0;
				set_status ("Press OK to see the answer");
				return 1;
			}
			reset_state ("Nothing more in Final drill queue");
			break;
		default:
			FXMessageBox::information (this, MBOX_OK, "Delete current", 
				"Cannot delete current element in actual learning state");
			break;
	}
	return 1;
}

long ma_window::on_delete (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::information (this, MBOX_OK, "Delete an element", 
			"If you want to delete an element, first you have to finish, "\
			"or stop (by pressing STOP), current learning.");
		return 1;
	}
	qtxt->setText  ("");
	atxt->setText  ("");
	set_status ("Type part of the question and/or the "
			"answer in q/a areas, and press \"OK\".");
	atxt->setEditable (true);
	qtxt->setEditable (true);
	state = 7;
	return 1;
}

long ma_window::on_stats (FXObject*, FXSelector, void*) {
	char s[4096];
	struct ma_stats_st fxs;
	ma_stats (&fxs);

	if (fxs.total_items == 0) {
		sprintf (s, "No elements in the learning process.");
		FXMessageBox::information (this, MBOX_OK, "Statistics", s);
		return 1;
	}
	sprintf (s, "First run: %s"
		"Items: %lu\n"
		"Days: %hu\n"
		"New items per day: %.2f\n\n"
		
		"Intervals, taking data from the last repetition of every item:\n"
		"Max interval: by ANN %hu, real: %hu days\n"
		"Min interval: by ANN %hu, real: %hu days\n"
		"Mean interval: by ANN %hu, real: %hu days\n\n"

		"Current max/min/mean interval computed by ANN: %hu/%hu/%hu days\n\n"
		
		"Not counting reviews ahead-of-time (e.g. Final drill):\n"
		"Sum of all reviews: %hu\n"
		"Max number of reviews of an item so far: %hu\n"
		"Mean number of repetitions: %.2f\n"
		"Mean grade: %.2f\n"
		"Aproximate retention: %.1f percent\n\n"
		
		"ANN TestError: %.3f\n"
		"Cases to train ANN: %lu (from user: %lu)",
		
		ctime(&(fxs.first_run)),
		fxs.total_items,
		fxs.days,
		fxs.item_per_day,

		fxs.max_l_ivl_ann, fxs.max_l_ivl_real,
		fxs.min_l_ivl_ann, fxs.min_l_ivl_real,
		fxs.mean_l_ivl_ann, fxs.mean_l_ivl_real,

		fxs.max_ivl,
		fxs.min_ivl,
		fxs.mean_ivl,

		fxs.sum_reviews,
		fxs.max_reviews,
		fxs.mean_reviews,
		fxs.mean_grade,

		fxs.retention,

		fxs.ann_error,
		fxs.ann_cases,
		fxs.user_ann_cases);

	FXMessageBox::information (this, MBOX_OK, "Basic statistics", s);
	return 1;
}

long ma_window::on_show_schedule (FXObject*, FXSelector, void*) {
	char s[4096];
				/* TODO: graph */

	sprintf (s, "Items scheduled for review for next x day:\n"
		"0) today: %hu items\n"
		"1) tomorrow: %hu items\n"
		"2) %hu items\n"
		"3) %hu items\n"
		"4) %hu items\n"
		"5) %hu items\n"
		"6) %hu items\n"
		"7) %hu items\n"
		"8) %hu items\n"
		"9) %hu items\n"
		"10) %hu items\n"
		"11) %hu items\n"
		"12) %hu items\n"
		"13) %hu items\n"
		"14) %hu items\n"
		"Items scheduled for review in 15-60 days: %hu\n",
		ma_n_of_rpts_for_today(),
		ma_rpts_for (1,1),
		ma_rpts_for (2,2),
		ma_rpts_for (3,3),
		ma_rpts_for (4,4),
		ma_rpts_for (5,5),
		ma_rpts_for (6,6),
		ma_rpts_for (7,7),
		ma_rpts_for (8,8),
		ma_rpts_for (9,9),
		ma_rpts_for (10,10),
		ma_rpts_for (11,11),
		ma_rpts_for (12,12),
		ma_rpts_for (13,13),
		ma_rpts_for (14,14),
		ma_rpts_for (15,60));

	FXMessageBox::information (this, MBOX_OK, "Schedule", s);
	return 1;
}

long ma_window::on_open_another (FXObject*, FXSelector, void*) {
	FXString folder;
	FXDirDialog opendialog (this, "Open another database");
	folder = path_to_data_dir;
	opendialog.setDirectory (folder);
	if (!opendialog.execute())
		return 1;
	folder = opendialog.getDirectory ();

	ma_deinit();
	int i = ma_init (folder.text());
	if (i & DIR_NOT_ACCESSIBLE) { /* if no files with database has been found */
		FXMessageBox::error (this, MBOX_OK, "Invalid directory",
			"This directory doesn't contain valid MemAid database.");
		ma_init (path_to_data_dir);
		return 1;
	}

	if (i & MEMAID_ALREADY_RUNNING) {
		FXMessageBox::error (this, MBOX_OK, "Database locked",
			"This database is locked, I cannot open it.\n"
			"Is another MemAid copy running on it?\n"
			"If you are sure that not (e.g. previous MemAid copy crashed on it and left database locked)\n"
			"Then you can manually delete dumbfile from the directory.");
		ma_init (path_to_data_dir);
	}
	else {
		if (i & ASCII_DATA_DOES_NOT_EXIST)
			FXMessageBox::information (this, MBOX_OK, "New database",
				"This directory doesn't contain any valid MemAid database, I will create new one.");
		else
			greeting();
	}
	return 1;
}

class ldialog : public FXDialogBox {
	FXDECLARE(ldialog)

	bool edit;
	int items;
	FXList *l;
	FXHorizontalFrame *contents;
	FXVerticalFrame *f;
	FXVerticalFrame *button_frame;
public:
	enum {
		ID_EDIT_EL=FXDialogBox::ID_LAST,
		ID_DELETE_EL, // ID_DELETE is already used!
		ID_LAST
	};
	void put_elements(void);
	long on_edit (FXObject*, FXSelector, void*);
	long on_delete (FXObject*, FXSelector, void*);

	ldialog () {}
	ldialog (FXWindow *wi) : FXDialogBox (wi,"List",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE) {
		items = 0;
		edit = false;
		contents = new FXHorizontalFrame (this, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y,
			0,0,0,0,0,0,0,0);

		f = new FXVerticalFrame (contents,
			FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT,
			0,0,0,0,10,10,10,10);

		l = new FXList
			(f, 0,NULL,0,LIST_BROWSESELECT|HSCROLLER_NEVER|LAYOUT_FIX_WIDTH|LAYOUT_FILL_Y,
			 0,0,300,250);

		button_frame = new FXVerticalFrame (contents,
			FRAME_SUNKEN | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);

		new FXButton (button_frame, "Close\t\tCloses this dialog", NULL,
				this, ID_CANCEL, FRAME_THICK | FRAME_RAISED |
				LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);

		new FXButton (button_frame, "Edit\t\tAllows you to edit selected element", NULL,
				this, ID_EDIT_EL, FRAME_THICK | FRAME_RAISED |
				LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);

		new FXButton (button_frame, "Delete\t\tDeletes currently selected element", NULL,
				this, ID_DELETE_EL, FRAME_THICK | FRAME_RAISED |
				LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0,0,0,0,10,10,10,10);
		put_elements();
	}
	~ldialog () {
		if (edit == false)
			main_win->greeting();
	}
};

void ldialog::put_elements (void) {
	struct elem *li = ma_iterate_list(NULL);
	const int mqsl = 32; // max question show len
	char qc[mqsl+5];

	l->clearItems ();
	items = 0;
	while (li != NULL) {
		bool fit = strlen (li->q) < mqsl - 1; 
		if (fit == true) 
			strcpy (qc, li->q);
		else {
			strncpy (qc, li->q, mqsl - 1);
			qc[mqsl-1] = '\0';
			strcat (qc, " ...");
		}
		l->appendItem (qc);
		li = li->nxt;
		++items;
	}
}

long ldialog::on_edit (FXObject*, FXSelector, void*) {
	int current = 0, aim = l->getCurrentItem();
	el = ma_iterate_list (NULL);
	while (current != aim) {
		el = el->nxt;
		if (el == NULL) {
			ma_error ("ldialog::on_edit(): FATAL ERROR\n");
			ma_deinit();
			exit (1);
		}
		++current;
	}
	main_win->qtxt->setText (el->q);
	main_win->atxt->setText (el->a);
	main_win->qtxt->setEditable (true);
	main_win->atxt->setEditable (true);
	edit = true;
	state = 6;
	ok_pressed = 1;
	hint = 0;
	main_win->set_status ("Please correct question and/or answer, "
				"then press \"OK\" button.");
	getApp()->stopModal (this, FALSE);
	hide();
	return 1;
}

long ldialog::on_delete (FXObject*, FXSelector, void*) {
	if (MBOX_CLICKED_NO == FXMessageBox::question (this, MBOX_YES_NO, "Delete selected element",
			"Are you sure you want delete selected element?"))
		return 1;
	if (items == 0) {
		FXMessageBox::error (this, MBOX_OK, "No element", "No element to delete");
		return 1;
	}
	int current = 0, aim = l->getCurrentItem();
	struct elem *li = ma_iterate_list(NULL);
	while (current != aim) {
		li = li->nxt;
		if (li == NULL) {
			ma_error ("ldialog::on_delete(): FATAL ERROR\n");
			ma_deinit();
			exit (1);
		}
		++current;
	}
	ma_delete_el (li);
	put_elements ();
	return 1;
}


FXDEFMAP(ldialog) ldialog_map[]={
	//________Message_Type_____________________ID_______________Message_Handler_______
//	FXMAPFUNC(SEL_COMMAND,		ldialog::ID_QUIT,		ldialog::on_close),
	FXMAPFUNC(SEL_COMMAND,		ldialog::ID_DELETE_EL,		ldialog::on_delete),
	FXMAPFUNC(SEL_COMMAND,		ldialog::ID_EDIT_EL,		ldialog::on_edit),
//	FXMAPFUNC(SEL_KEYPRESS,		ldialog::ID_KEY_PRESS,		ldialog::on_key_press)
};

FXIMPLEMENT(ldialog,FXDialogBox,ldialog_map,ARRAYNUMBER(ldialog_map))


long ma_window::on_list (FXObject*, FXSelector, void*) {
	if (state != 0) {
		FXMessageBox::information (this, MBOX_OK, "List",
			"Cannot list elements while you learn."
			" Please stop current learning by pressing STOP button.");
			// if you would like to override it:
			// keep in mind problems with struct elem *el pointer!
		return 1;
	}
	if (ma_iterate_list (NULL) == NULL) {
		FXMessageBox::information (this, MBOX_OK, "List",
			"Nothing to list, you don't have any items in your learning process..");
		return 1;
	}
	ldialog d (this);
	d.execute();
	return 1;
}


int main (int argc, char *argv[]) {
	int i;
	
	path_to_data_dir = NULL;

	for (i = 1; i < argc; i++) {
		if (argv[i][0] == 'd')
			path_to_data_dir = &argv[i][1];
	}
	if (path_to_data_dir != NULL)
		i = ma_init (path_to_data_dir);
	else
		i = ma_init ("data/");
	if (i & DIR_NOT_ACCESSIBLE || i & MEMAID_ALREADY_RUNNING)
		return 1;

	application = new FXApp ("MemAid", "memaid");

	application->init (argc, argv);
	main_win = new ma_window (application);
	application->create();
	application->run();

	delete main_win;
	return 0;
}

