#include <fx.h>
#include <fxkeys.h>
#include <FXFont.h> 
#include <FXMenubar.h>
#include <FXMessageBox.h> 
#include <FXStatusbar.h>
#include <FXStatusline.h>
#include <FXString.h> 
#include <FXText.h> 
#include "core/memaid.h"
#include "defs.h"

#ifndef MA_WINDOW_H
#define MA_WINDOW_H

#define GREETING_TXT "MemAid " MEMAID_VERSION "\n\n"

#define MEMAID_QUOTE1 "\"Minimum time, maximum effect.\""
#define MEMAID_QUOTE2 "\"Learning how to learn is life's most important skill.\" - Tony Buzan"
#define MEMAID_QUOTE3 "\"They know enough who know how to learn.\" - Henry Adams"

#define MEMAID_MANUAL "Full manual can be found in MemAid package in docs/ dir, "\
"or on http://memaid.sf.net/docs/\n\n"\
"Knowledge is stored in items (\"question and answer\" format).\n"\
"Whatever information you want to memorize, think about a question and answer relevant to it,\n"\
"and then input this question-answer pair to MemAid (\"Input new items\" from the \"Learn menu\").\n"\
"Then choose a grade: how well you remember it, "\
"so MemAid can compute next optimal repetition date.\n\n"\
"When you finish entering items, press the \"Stop\" button.\n"\
"All facts you input in this way will be in the learning process -\n"\
"MemAid will take care you remember them with minimum time spend on repetitions.\n\n"\
"If you run MemAid later (after a day or more), "\
"you will have some scheduled repetitions.\n"\
"MemAid will show you a question. Try to recall the answer.\n"\
"Then click \"OK\" and grade yourself - "\
"how well did you remember this fact/answer.\n"\
"This grade (and other data MemAid stores) will allow MemAid to schedule next repetition\n"\
"on a date when you will be close to forgetting it. "\
"This is the most optimal date -\n"\
"you shouldn't waste your time on repetitions of things you remember well, should you?"

#define MEMAID_TIPS "When you think about an answer, you can point your mouse to answer area and\n"\
"press the key/character you think is the next character in hidden part of answer.\n"\
"If you are right - this letter will show up.\n\n"\
"If focus is on question, you can use keyboard shortcuts:\n"\
"0-5 are grades,\n"\
"Space and Enter work like \"OK\" button.\n\n"\
"You can edit some variables related to how MemAid looks like (fonts, colors, etc.)\n"\
"see .foxrc/memaid/memaid in your home directory.\n\n"\
"See http://memaid.sourceforge.net/docs/tips.html for more hints and tips"

#define GRADE_THRESHOLD 3 /* used in Final Drill */

/* used by "hint" feature; how many percent of answer is showed (0-100%) */
static unsigned short hint = 0;
static struct elem* el = NULL;
static char* path_to_data_dir = NULL;
static char q[MAX_Q_LEN], a[MAX_A_LEN];
static class FXApp* application;
static class ma_window *main_win;
static char ok_pressed = 0; 

class ma_window : public FXMainWindow {
	FXDECLARE(ma_window)

private:
	FXTimer			*timerhandle;
	FXHorizontalFrame	*contents;
	FXVerticalFrame		*button_frame;
	FXVerticalFrame		*qa_frame;
	FXStatusbar		*status_bar;
	FXStatusline		*status_line;
	FXMenubar		*menu_bar;
	FXMenuPane		*file_menu, *edit_menu, *learn_menu, *stats_menu, *about_menu;
	bool edit_current;
public:
	FXText			*qtxt,	*atxt;

	ma_window(){} // just to make a macro happy; we will never use default constructor
	~ma_window() {
		ma_deinit();
	}

public:
	enum {
		ID_CANVAS=FXMainWindow::ID_LAST,
		ID_Q_TXT,
		ID_A_TXT,
		ID_OPEN,
		ID_LIST,
		ID_EXPORT,
		ID_EDIT_IT,
		ID_EDIT_CURRENT,
		ID_DELETE,
		ID_DELETE_CURRENT,
		ID_LEARN,
		ID_LEARN_NEW,
		ID_LEARN_NFQ,
		ID_FORCE_RPT,
		ID_FINAL_DRILL,
		ID_SHOW_ABOUT,
		ID_SHOW_LICENSE,
		ID_STOP,
		ID_HINT,
		ID_STATS,
		ID_TIPS,
		ID_SHOW_SCHEDULE,
		ID_OK,
		ID_GRADE_0, ID_GRADE_1, ID_GRADE_2, ID_GRADE_3, ID_GRADE_4, ID_GRADE_5,
		ID_KEY_PRESS,
		ID_TRAIN_ANN,
		ID_HELP,
		ID_LAST
	};
	ma_window (FXApp* a);
	virtual void create() {
		FXMainWindow::create();
		show(PLACEMENT_SCREEN);
	}
	void show_q  (const char *s, bool edit=false) {
		if (edit == true)
			qtxt->setText(s);
		else {
			char t[MAX_Q_LEN];
			strcpy (t, s);
			ma_remove_tags (t);
			qtxt->setText (t);
			ma_run_tags (s, GUI_VERSION);
			atxt->setText ("");
		}
	}
	void show_a (const char *s, bool edit=false) {
		if (edit == true) 
			atxt->setText(s);
		else {
			char t[MAX_A_LEN];
			strcpy (t, s);
			ma_remove_tags (t);
			atxt->setText (t);
			ma_run_tags (s, GUI_VERSION);
		}
	}
	void set_status (char* s) {
		status_line->setNormalText (s);
	}
	
	int key_hint (long key);
	void greeting(void);
	void add_to_status (char*);
	long on_ok_button (FXObject*, FXSelector, void*);
	void on_ok (void);
	void reset_state (char* = NULL, char* = NULL, char* = NULL);
	void after_grade (u_short grade);
	long on_train_ann      (FXObject*, FXSelector, void*) {
		ma_train_ann_for_x_secs (1);
		return 1;
	}
	long on_open_another	(FXObject*, FXSelector, void*);
	long on_list		(FXObject*, FXSelector, void*);
	long on_learn		(FXObject*, FXSelector, void*);
	long on_learn_new	(FXObject*, FXSelector, void*);
	long on_learn_nfq	(FXObject*, FXSelector, void*);
	long on_final_drill	(FXObject*, FXSelector, void*);
	long on_force_rpt	(FXObject*, FXSelector, void*);
	long on_stop		(FXObject*, FXSelector, void*);
	long on_export		(FXObject*, FXSelector, void*);
	long on_delete		(FXObject*, FXSelector, void*);
	long on_edit		(FXObject*, FXSelector, void*);
	long on_edit_current	(FXObject*, FXSelector, void*);
	long on_delete_current	(FXObject*, FXSelector, void*);
	long on_hint		(FXObject*, FXSelector, void*);
	long on_stats		(FXObject*, FXSelector, void*);
	long on_show_schedule	(FXObject*, FXSelector, void*);
	long show_help		(FXObject*, FXSelector, void*);
	long show_tips		(FXObject*, FXSelector, void*);
	
	void update_current (struct elem* e) {
		if (edit_current == false)
			return;
		atxt->setEditable (false);
		qtxt->setEditable (false);
		
		FXString qs = qtxt->getText();
		FXString as = atxt->getText();

		e->q = (char*) realloc (e->q, (strlen(qs.text()) + 1) * sizeof(char));
		e->a = (char*) realloc (e->a, (strlen(as.text()) + 1) * sizeof(char));
		
		strcpy (e->q, qs.text());
		strcpy (e->a, as.text());
		
		edit_current = false;
		return;
	}
	long on_key_press (FXObject* o, FXSelector, void *ptr) {
		if (qtxt->isEditable())
			return 0;
		FXEvent* event = (FXEvent*) ptr;
		if (o == atxt && ok_pressed == 0) {
			if (key_hint ((long)event->code))
				return 1;
			return 0;
		}
			
		switch (event->code) {
			case KEY_KP_0:
			case KEY_0:
				after_grade (0);
				return 1;
			case KEY_KP_1:
			case KEY_1:
				after_grade (1);
				return 1;
			case KEY_KP_2:
			case KEY_2:
				after_grade (2);
				return 1;
			case KEY_KP_3:
			case KEY_3:
				after_grade (3);
				return 1;
			case KEY_KP_4:
			case KEY_4:
				after_grade (4);
				return 1;
			case KEY_KP_5:
			case KEY_5:
				after_grade (5);
				return 1;
			case KEY_KP_Enter:
			case KEY_space:
				on_ok();
				return 1;
			default:
				return 0;
		}
	}
	long show_about   (FXObject*, FXSelector, void*);
	long show_license (FXObject*, FXSelector, void*);
	long grade_0 (FXObject*, FXSelector, void*) {
		after_grade(0);
		return 1;
	}
	long grade_1 (FXObject*, FXSelector, void*) {
		after_grade(1);
		return 1;
	}
	long grade_2 (FXObject*, FXSelector, void*) {
		after_grade(2);
		return 1;
	}
	long grade_3 (FXObject*, FXSelector, void*) {
		after_grade(3);
		return 1;
	}
	long grade_4 (FXObject*, FXSelector, void*) {
		after_grade(4);
		return 1;
	}
	long grade_5 (FXObject*, FXSelector, void*) {
		after_grade(5);
		return 1;
	}
};

#endif /* MA_WINDOW_H */

