/*
   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA
*/

#include <ctype.h>
#include "core/memaid.h"
#include "defs.h"

#define CONSOLE_WIDTH 80

static int terminal_width;

static char *simple_stats (void) {
	static char scrap[256];
	int i, j;

	i = ma_list_n_of_els();
	j = ma_n_of_rpts_for_today();

	if (ma_iterate_list (NULL) == NULL) {
		strcpy (scrap, "Learning process hasn't started yet.");
		return scrap;
	}
	if (i > 1) {
		switch (j) {
		case 0:
			sprintf (scrap,
				"%d elements in the learning process. No repetitions left for today.", i);
			break;
		case 1:
			sprintf (scrap,
				"%d elements in the learning process. 1 repetition left for today.", i);
			break;
		default:
			sprintf (scrap,
				"%d elements in the learning process. %d repetitions left for today.", i, j);
			break;
		}
	}
	else { /* i == 1 */
		switch (j) {
		case 0:
			sprintf (scrap,
				"1 element in the learning process. No repetition left for today.");
			break;
		case 1:
			sprintf (scrap,
				"1 element in the learning process. 1 repetition left for today.");
			break;
		default:
			sprintf (scrap,
				"Oops. A weird error in simple_stats()");
			break;
		}
	}
	return scrap;
}

static void put_n_char (char c, register int i) {
	if (i < 0)
		return;
	do {
		putchar (c);
	} while (--i);
	return;
}

static void print_centered (const char *s) {
	int i;
	i = strlen (s);

	put_n_char (' ', (terminal_width >> 1) - (i >> 1));
	printf ("%s", s);
	put_n_char (' ', (terminal_width >> 1) - (i >> 1));
	putchar ('\n');
	return;
}

static void show_nicely (char *pos, int q) {
	int len = 0;
	char begin = 1, *ps, t[MAX_A_LEN];
	int max = terminal_width;
	
	/* [ Question ]
	   123456789012
	   [ Answer ]
	   1234567890 */
	
	assert (strlen (pos) < MAX_A_LEN);
	strcpy (t, pos);		
	ma_remove_tags (t);
	ps = t;
	
	putchar (' ');
	if (q) {
		len = (max >> 1) - 7;
		put_n_char ('-', len);
		printf ("[ Question ]");
		put_n_char ('-', len);
	}
	else {
		len = (max >> 1) - 6;
		put_n_char ('-', len);
		printf ("[ Answer ]");
		put_n_char ('-', len);
	}
	putchar ('\n');

	while (*ps != '\0') {
		if (begin) {
			printf ("| ");
			begin = 0;
			len = 0;
		}
		if (*ps == '\n') {
			while (len++ <= max - 4)
				putchar (' ');
			printf ("|\n");
			begin = 1;
			ps++;
			continue;
		}
	
		putchar (*ps);
		ps++;
		if (++len >= max - 10){
			if (len == max - 4) {
				printf (" |\n");
				begin = 1;
			}
			else
				if (*ps == ' ' && (!strchr (ps+1, ' ') || strchr (ps+1, ' ') > ps + (max-len-4))) {
					while (len++ <= max - 4)
						putchar (' ');
					printf ("|\n");
					begin = 1;
				}
		}
	}
	while (len++ <= max - 4)
		putchar (' ');
	printf ("|\n");

	putchar (' ');
	put_n_char ('-', max - 2);
	putchar ('\n');
	ma_run_tags (pos, CONSOLE_VERSION);
	return;
}

static void add_material (void) {
	char *p, *q, *a;
	u_short grade;

	q = (char*) malloc (sizeof(char) * (MAX_Q_LEN + MAX_A_LEN));
	a = q + sizeof(char) * MAX_Q_LEN;

	puts ("Type your question:");
	fgets (q, MAX_Q_LEN, stdin);
	if ((p=strchr (q, '\n'))) *p = '\0';

	puts ("Answer:");
	fgets (a, MAX_A_LEN, stdin);
	if ((p=strchr (a, '\n'))) *p = '\0';

	do {
		puts ("How well do you remember this fact now? [5=best, 0=worst]");
		scanf ("%hu", &grade);
		while (getchar() != '\n')
			continue;
	} while (grade > 5);

	ma_make_new_el (grade, q, a);

	free (q);
	return;
}

/*
static void delete_element (void) {
	puts ("Type full element question to delete:");
	fgets (scrap, 128, stdin);
	if (strchr (scrap, '\n')) *strchr (scrap, '\n') = '\0';
	if (delete_el (scrap))
		puts ("There is no such element.\n");
	else
		puts ("OK. Element deleted.\n");
	return;
} */

static int repeat (void) {
	u_short grade = 10;
	struct elem *e;
	
	while ((e = ma_el_to_repeat ()) != NULL) {
		show_nicely (e->q, 1);
		puts ("Press \'Enter\' to see the answer (or type anything other to stop)");
	
		/* train ANN a bit while user think about an answer */
		ma_train_ann_for_x_secs(2);
		
		if (getchar() != '\n') {
			while (getchar() != '\n')
				continue;
			return 2;
		}

		show_nicely (e->a, 0);
	
		do {
			puts ("Grade? [5=best, 0=worst]");
			ma_train_ann_for_x_secs(2);
			scanf ("%hu", &grade);
			while (getchar() != '\n')
				continue;
		} while (grade > 5);
		
		ma_move_el (e, grade);	
	}
	if (grade == 10) /* there were no elements to repeat */
		return 1;
	return 0;
}

static int learn_new_stuff(void) {
	u_short grade;
	char *q, *a;

	q = (char*) malloc (sizeof(char) * (MAX_Q_LEN+MAX_A_LEN));
	a = q + sizeof(char) * MAX_Q_LEN;

	while (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
		show_nicely (q, 1);
		puts ("Press \'Enter\' key to see the answer");
		getchar();
		show_nicely (a, 0);
		do {
			puts ("Grade? [5=best, 0=worst]");
			scanf ("%hu", &grade);
			while (getchar() != '\n')
				continue;
		} while (grade > 5);

		ma_make_new_el (grade, q, a);
		ma_get_new_el (MA_DELETE_IT, q, a); /* deletes element from queue.txt */
	}

	free (q);
	return 0;
}

extern float TestError;

static void print_greet(void) {
	int i;
	
	putchar ('\n');
	
	i = (terminal_width >> 1) - 28;
	put_n_char (' ', i);
	puts(" ***   ***    ****   ***   ***    *****   **  *****");
	put_n_char (' ', i);
	puts ("** ** ** **  **     ** ** ** **  **   **      **  **");
	put_n_char (' ', i);
	puts ("**  ***  **  ****   **  ***  **  *******  **  **  **");
	put_n_char (' ', i);
	puts ("**   *   **  **     **   *   **  ** * **  **  **  **");
	put_n_char (' ', i);
	puts ("**       **  **     **       **  **   **  **  **  **");
	put_n_char (' ', i);
	puts ("**       **   ****  **       **  **   **  **  *****");
	
	i = (terminal_width >> 1) - 13;
	i >>= 1;
	put_n_char (' ', i);
	put_n_char ('-', i);
	fputs ("[ console version " MEMAID_VERSION " ]", stdout);
	put_n_char ('-', i);
	put_n_char (' ', i);
	putchar ('\n');
	return;
}

static void print_about(void) {
	puts ("MemAid is like a traditional flash-card program to help you memorise question/answer pairs,\n"
		"but with an important twist:\n"
		"it uses a neural network to schedule the best time for an item to come up for review.\n"
		"Difficult items that you tend to forget quickly will be scheduled more often,\n"
		"while MemAid won't waste your time on things you remember well.\n"
		"This means your learning process becomes much more efficient,\n"
		"also because the neural network gradually adapts to your personal memory model.\n");
	
	puts ("\thttp://memaid.sf.net");
	
	puts (MEMAID_LICENSE);
	putchar ('\n');	
	return;
}

static void print_usage (void) {
	printf ("Usage:\n");
	printf ("memaid-console tN/d/lN\n");
	printf ("tN = run console version with terminal width set to N (default is 80)\n");
	printf ("d/a/path/data/dir = run console version with specific path to data dir, "\
		"instead of default\n");	
	printf ("lN = train ANN for N number of seconds, then quit.\n");
	return;
}

int main (int argc, char *argv[]) {
	char c, d;
	u_short i, j, k, l, arg;
	char *path_to_data_dir = NULL;

/* Compiled with -fbounds-checking, commented out following code gives
me:
"Bounds error:
	in strchr, string argument is a string with size 35 overrunning
	the end of the object's allocated memory."

				on FreeBSD-5.1p2.
				What's wrong?
   
   if (setlocale (LC_ALL, "") == NULL) 
		ma_error ("Cannot set \"locale\"\n");
*/

	terminal_width = CONSOLE_WIDTH;
	for (arg = 1, k = 0; arg < argc; arg++) {
		switch (argv[arg][0]) {
			case 'd':
				path_to_data_dir = &argv[arg][1];
				break;
			case 'l':
				k = atoi (&argv[arg][1]);
				break;
			case 't':
				terminal_width = atoi (&argv[arg][1]);
				break;
			default:
				print_usage();
				return 0;
		}
	}
	if (path_to_data_dir != NULL) 
		i = (u_short) ma_init(path_to_data_dir);
	else
		i = (u_short) ma_init("data/");
	if (i & DIR_NOT_ACCESSIBLE)
		exit(1); /* ma_init() has already printed error */

	if (i & MEMAID_ALREADY_RUNNING)
		exit(1); /* ma_init() has already printed error */

	if (k) {
		ma_train_ann_for_x_secs(k);
		ma_deinit();
		return 0;
	}
	print_greet();
	print_centered (simple_stats());
	
	while (1) {
		print_centered ("[L]earn, [A]dd element, [T]est NN, A[b]out, [Q]uit");
		c = getchar();  d = getchar();
		if (isalnum(d) && !isalnum(c)) {  c = d;  getchar();  }
		if (c == 'l' || c == 'L')  break;
		if (c == 'a' || c == 'A') {  add_material();  continue;  }
		if (c == 'b' || c == 'B') {  print_about(); continue; }
/*		if (c == 'd' || c == 'D') {  delete_element();  continue;  } */
		if (c == 'q' || c == 'Q') goto end;
		if (c != 't' && c != 'T') continue;
		
		puts ("real interval? (0-2048)\n"\
				"(Hint: how many days since you last time repeated that item?\n"\
				"0 if this is your first time when you see that item)");
		scanf ("%hu", &i);
		getchar();
		assert (i <= MA_MAX_N_OF_RPTS);

		puts ("planned by ANN interval? (0-2048)\n"\
				"(Hint: the same as real, unless repetition wasn't on time)");
		scanf ("%hu", &j);
		getchar();
		assert (j <= MA_MAX_N_OF_RPTS);
		
		puts ("how many repetitions they were? (0-128)");
		scanf ("%hu", &k);
		getchar ();
		assert (k <= MA_MAX_INTERVAL);
		
		puts ("current grade? (5=best, 0=worst)");
		scanf ("%hu", &l);
		getchar ();
		assert (l <= 5);
		
		printf ("New interval = %hu days.\n", ma_new_interval (i, j, k, l));
	}
		
	switch (repeat()) {
		case 0:
			puts ("Nothing to repeat.");
			break;
		case 1:
			puts ("Nothing more to repeat.");
			break;
		case 2:
			puts ("You stopped repeating elements.");
			break;
		default:
			ma_error ("Whoa. This error should never happen...\n");
	}

	while (1) {
		puts ("Would you like to learn new stuff? [y/n]");
		scanf ("%c", &c);
		if (c == 'n' || c == 'N') {
			puts ("OK");
			goto end;
		}
		if (c != 'y' && c != 'Y') {
			puts ("I can understand only \'y\' and \'n\'");
			continue;
		}
		break;
	}

	learn_new_stuff();
	puts ("Nothing more to learn");
end:
	ma_deinit();
	return 0;
}


